@@ -1,9 +1,9 @@
 #!/bin/bash
 
-source $MOD_GIT_ROOTDIR/doc/example/compiler-etc-dependencies/example-sessions/common_code.sh
+source "$MOD_GIT_ROOTDIR"/doc/example/compiler-etc-dependencies/example-sessions/common_code.sh
 
 do_cmd module purge
-do_cmd module load $GCCGNU/9.1.0
+do_cmd module load "$GCCGNU/9.1.0"
 case $TMP_STRATEGY in
    flavours|homebrewed)
 	do_cmd module load simd/avx2
@@ -22,7 +22,7 @@ do_cmd bar
 do_cmd module unload bar
 case $TMP_STRATEGY in
    flavours|homebrewed)
-	do_cmd module switch $AUTOFLAG simd simd/avx
+	do_cmd module switch "$AUTOFLAG" simd simd/avx
 	do_cmd module load bar/5.4
 	;;
    modulerc)
@@ -39,7 +39,7 @@ do_cmd bar
 do_cmd module unload bar
 case $TMP_STRATEGY in
    flavours|homebrewed)
-	do_cmd module switch $AUTOFLAG simd simd/sse4.1
+	do_cmd module switch "$AUTOFLAG" simd simd/sse4.1
 	do_cmd module load bar/5.4
 	;;
    modulerc)
