@@ -124,13 +124,7 @@ proc module-info {what {more {}}} {
          # refresh mod name version and variant to correctly get all matching
          # tags (in case tags apply to specific module variant)
          set modname [currentState modulename]
-         set vrlist [getVariantList $modname 1]
-         if {[llength $vrlist]} {
-            lassign [parseModuleSpecification 0 0 0 0 $modname {*}$vrlist]\
-               modnamevr
-         } else {
-            set modnamevr $modname
-         }
+         set modnamevr [getAndParseLoadedModuleWithVariant $modname]
          collectModuleTags $modnamevr
 
          if {$more ne {}} {
@@ -499,6 +493,10 @@ proc module-hide {args} {
    }
 }
 
+proc hide-modulefile {modfile} {
+   module-hide $modfile
+}
+
 proc module-tag {args} {
    # parse application criteria arguments to determine if command apply
    lassign [parseApplicationCriteriaArgs 0 0 {*}$args] apply isnearly after\
@@ -828,6 +826,9 @@ proc putsModfileCmd {itrp args} {
             stderr {
                set reportArgs [list [lindex $args 1]]
             }
+            log {
+               set logPuts [lindex $args 1]
+            }
          }
       }
       3 {
@@ -842,6 +843,9 @@ proc putsModfileCmd {itrp args} {
                stderr {
                   set reportArgs [list [lindex $args 2] 1]
                }
+               log {
+                  set logPuts [lindex $args 2]
+               }
             }
          } else {
             set wrongNumArgs 1
@@ -856,6 +860,9 @@ proc putsModfileCmd {itrp args} {
    # not to confuse people with an error reported by an internal name (_puts)
    if {[info exists wrongNumArgs]} {
       knerror {wrong # args: should be "puts ?-nonewline? ?channelId? string"}
+   # send content to log system
+   } elseif {[info exists logPuts]} {
+      log $logPuts
    # defer puts if it targets stdout (see renderSettings)
    } elseif {[info exists deferPuts]} {
       # current module is qualified for refresh evaluation
@@ -1020,6 +1027,9 @@ proc conflict {args} {
    }
    defineModEqProc [isIcase] [getConf extended_default]
 
+   set conflict_unload [expr {[getConf conflict_unload] && [getConf\
+      auto_handling]}]
+
    # parse module version specification
    set args [parseModuleSpecification 0 0 0 0 {*}$args]
 
@@ -1027,6 +1037,7 @@ proc conflict {args} {
 
    foreach mod $args {
       set is_conflict_loading 0
+      set unload_attempt 0
       set loaded_conflict_mod_list [getLoadedMatchingName $mod returnall]
 
       if {![llength $loaded_conflict_mod_list]} {
@@ -1038,14 +1049,29 @@ proc conflict {args} {
             set loaded_conflict_mod_list [getLoadedMatchingName $mod\
                returnall 1]
             if {[llength $loaded_conflict_mod_list]} {
+               # no conflict unload attempt on loading module
                set is_conflict_loading 1
             }
          }
+      } elseif {$conflict_unload} {
+         set still_loaded_conflict_mod_list {}
+         # unload attempt in reverse load order
+         foreach loaded_conflict_mod [lreverse $loaded_conflict_mod_list] {
+            if {[cmdModuleUnload conun match 1 s 0 $loaded_conflict_mod]} {
+               lappend still_loaded_conflict_mod_list $loaded_conflict_mod
+            }
+         }
+         set loaded_conflict_mod_list $still_loaded_conflict_mod_list
+         set unload_attempt 1
       }
 
       if {[llength $loaded_conflict_mod_list]} {
-         reportPresentConflictError $curmodnamevr $loaded_conflict_mod_list\
-            $is_conflict_loading
+         setConflictErrorAsReported {*}$loaded_conflict_mod_list
+
+         # error msg has already been sent if a conflict unload was attempted
+         set msg [expr {$unload_attempt ? {} : [getPresentConflictErrorMsg\
+            $curmodnamevr $loaded_conflict_mod_list $is_conflict_loading]}]
+         knerrorOrWarningIfForced $msg MODULES_ERR_GLOBAL
       }
    }
 
@@ -1061,6 +1087,7 @@ proc registerCurrentModuleConflict {args} {
 
 proc parsePrereqCommandArgs {cmd args} {
    set tag_list {}
+   set modulepath_list {}
    set optional 0
    set opt_list {}
    set prereq_list {}
@@ -1072,6 +1099,14 @@ proc parsePrereqCommandArgs {cmd args} {
          set tag_list [split $arg :]
          lappend opt_list $arg
          unset nextargistaglist
+      } elseif {[info exists nextargismodulepathlist]} {
+         set modulepath_list {}
+         # record list of absolute paths
+         foreach modulepath [split $arg :] {
+            lappend modulepath_list [getAbsolutePath $modulepath]
+         }
+         lappend opt_list $arg
+         unset nextargismodulepathlist
       } else {
          switch -glob -- $arg {
             --optional {
@@ -1089,6 +1124,10 @@ proc parsePrereqCommandArgs {cmd args} {
                set nextargistaglist 1
                lappend opt_list $arg
             }
+            --modulepath {
+               set nextargismodulepathlist 1
+               lappend opt_list $arg
+            }
             -* {
                knerror "Invalid option '$arg'"
             }
@@ -1110,16 +1149,16 @@ proc parsePrereqCommandArgs {cmd args} {
    }
    if {![llength $prereq_list]} {
       knerror "wrong # args: should be \"$cmd ?--optional? ?--tag? ?taglist?\
-         modulefile ?...?\""
+         ?--modulepath? ?modulepathlist? modulefile ?...?\""
    } elseif {[set mispopt [lsearch -inline -glob $prereq_list --*]] ne {}} {
       knerror "Misplaced option '$mispopt'"
    }
-   return [list $tag_list $optional $opt_list $prereq_list]
+   return [list $tag_list $modulepath_list $optional $opt_list $prereq_list]
 }
 
 proc prereqAnyModfileCmd {tryload auto args} {
-   lassign [parsePrereqCommandArgs prereq {*}$args] tag_list optional\
-      opt_list args
+   lassign [parsePrereqCommandArgs prereq {*}$args] tag_list modulepath_list\
+      optional opt_list args
 
    set currentModule [currentState modulename]
    set curmodnamevr [currentState modulenamevr]
@@ -1138,19 +1177,36 @@ proc prereqAnyModfileCmd {tryload auto args} {
       }
       lappend record_list {*}$args
       setLoadedPrereq $currentModule $record_list
+
+      if {[llength $modulepath_list]} {
+         setLoadedPrereqPath $currentModule $record_list $modulepath_list
+      }
    }
 
    if {$auto} {
+      # convert modulepath list to keep entries not matching any enabled
+      # modulepaths and transform entries into all matching enabled modulepath
+      set converted_list {}
+      foreach modulepath $modulepath_list {
+         set matching_list [getMatchingModulepathList $modulepath]
+         if {[llength $matching_list]} {
+            appendNoDupToList converted_list {*}$matching_list
+         } else {
+            appendNoDupToList converted_list $modulepath
+         }
+      }
+      set modulepath_list $converted_list
+
       # try to load prereq as dependency resolving is enabled
       lassign [loadRequirementModuleList $tryload $optional $tag_list\
-         {*}$args] retlo prereqloaded
+         $modulepath_list {*}$args] retlo prereqloaded
    } else {
       set loadedmod_list {}
       foreach mod $args {
          # get all loaded or loading mod in args list
-         if {[set loadedmod [getLoadedMatchingName $mod returnfirst]] ne {}\
-            || [set loadedmod [getLoadedMatchingName $mod returnfirst 1]] ne\
-            {}} {
+         if {[set loadedmod [getLoadedMatchingName $mod returnfirst 0 {}\
+            $modulepath_list]] ne {} || [set loadedmod [getLoadedMatchingName\
+            $mod returnfirst 1 {} $modulepath_list]] ne {}} {
             lappend loadedmod_list $loadedmod
          }
       }
@@ -1160,7 +1216,7 @@ proc prereqAnyModfileCmd {tryload auto args} {
    if {!$prereqloaded} {
       if {!$optional} {
          # error if requirement is not satisfied unless if optional
-         reportMissingPrereqError $curmodnamevr {*}$args
+         reportMissingPrereqError $curmodnamevr $modulepath_list {*}$args
       }
    } elseif {!$auto} {
       # apply missing tag to all loaded module found (already done when
@@ -1236,14 +1292,12 @@ proc system {args} {
    }
 
    if {[catch {exec >&@stderr $shell $shellarg [join $args]}]} {
-       # non-zero exit status, get it:
-       set status [lindex $::errorCode 2]
+      # non-zero exit status, get it:
+      return [lindex $::errorCode 2]
    } else {
-       # exit status was 0
-       set status 0
+      # exit status was 0
+      return 0
    }
-
-   return $status
 }
 
 # test at least one of the collections passed as argument exists
@@ -1329,7 +1383,7 @@ proc execShAndGetEnv {elt_ignored_list shell script args} {
          set varre {export (\S+?)=["']?(.*?)["']?$}
          set funcre {(\S+?) \(\)\s?\n?{\s?\n(.+?)\n}$}
          set aliasre {(\S+?)='(.*?)'$}
-         set varvalmap [list {\"} {"} \\\\ \\]
+         set varvalmap [list {\"} \" \\\\ \\]
          set alvalmap [list {'\''} ' {'"'"'} ']
       }
       bash {
@@ -1343,7 +1397,7 @@ proc execShAndGetEnv {elt_ignored_list shell script args} {
          set aliasre {alias (\S+?)='(.*?)'$}
          set compre {complete (.+?) (\S+?)$}
          set comprevar [list match value name]
-         set varvalmap [list {\"} {"} \\\\ \\]
+         set varvalmap [list {\"} \" \\\\ \\]
          set alvalmap [list {'\''} ']
          lappend shellopts --noprofile --norc
       }
@@ -1358,7 +1412,7 @@ proc execShAndGetEnv {elt_ignored_list shell script args} {
          set aliasre {alias (\S+?)='(.*?)'$}
          set compre {complete (.+?) (\S+?)$}
          set comprevar [list match value name]
-         set varvalmap [list {\"} {"} \\\\ \\]
+         set varvalmap [list {\"} \" \\\\ \\]
          set alvalmap [list {'\''} ']
          lappend shellopts --noprofile --norc
       }
@@ -1374,7 +1428,7 @@ proc execShAndGetEnv {elt_ignored_list shell script args} {
          set funcre {(\S+?)\(\) {\n?(.+?)}[;\n]?$}
          set aliasre {(\S+?)=\$?'?(.*?)'?$}
          set varvalmap [list {\'} ']
-         set alvalmap [list {\"} {"} {\\'} ' {\'} ' {\\\\} {\\}]
+         set alvalmap [list {\"} \" {\\'} ' {\'} ' {\\\\} {\\}]
       }
       zsh {
          ##nagelfar ignore +2 Found constant
@@ -1862,6 +1916,11 @@ proc source-sh-di {args} {
    }
 }
 
+proc isVariantNameValid {name} {
+   return [expr {![string is digit -strict $name] && [regexp\
+      {^[A-Za-z0-9_][A-Za-z0-9_-]*$} $name]}]
+}
+
 # parse arguments set on a variant modulefile command
 proc parseVariantCommandArgs {args} {
    set dflvalue {}
@@ -1905,7 +1964,7 @@ proc parseVariantCommandArgs {args} {
    if {![info exists name]} {
       knerror {No variant name specified}
    }
-   if {![string match {[A-Za-z0-9_]} [string index $name 0]]} {
+   if {![isVariantNameValid $name]} {
       knerror "Invalid variant name '$name'"
    }
    set values [lrange $args $i end]
@@ -1952,7 +2011,7 @@ proc variant {itrp args} {
 
    # search for variant specification (most right-positionned value wins)
    for {set i [expr {[llength $vrlist]-1}]} {$i >= 0} {incr i -1} {
-      lassign [lindex $vrlist $i] vrname vrisbool vrvalue
+      lassign [lindex $vrlist $i] vrname vrnot vrisbool vrvalue
       if {$vrname eq $name} {
          # translate value in boolean canonical form (0/1) if variant is bool
          if {$isboolean && [string is boolean -strict $vrvalue]} {
@@ -2043,8 +2102,8 @@ proc require-fullname {} {
 }
 
 proc prereqAllModfileCmd {tryload auto args} {
-   lassign [parsePrereqCommandArgs prereq-all {*}$args] tag_list optional\
-      opt_list args
+   lassign [parsePrereqCommandArgs prereq-all {*}$args] tag_list\
+      modulepath_list optional opt_list args
    # call prereq over each arg independently to emulate a prereq-all
    foreach arg $args {
       prereqAnyModfileCmd $tryload $auto {*}$opt_list $arg
@@ -2052,8 +2111,8 @@ proc prereqAllModfileCmd {tryload auto args} {
 }
 
 proc always-load {args} {
-   lassign [parsePrereqCommandArgs always-load {*}$args] tag_list optional\
-      opt_list args
+   lassign [parsePrereqCommandArgs always-load {*}$args] tag_list\
+      modulepath_list optional opt_list args
    # append keep-loaded tag to the list, second tag list in opt_list will take
    # over the initial list defined
    lappend tag_list keep-loaded
@@ -2230,6 +2289,15 @@ proc sourceModfileCmd {itrp filename} {
    interp eval $itrp info script [currentState modulefile]
 }
 
+proc lsb-release {what} {
+   return [switch -- $what {
+      id {getState lsb_id}
+      release {getState lsb_release}
+      codename {getState lsb_codename}
+      default {knerror "lsb-release $what not supported"}
+   }]
+}
+
 # ;;; Local Variables: ***
 # ;;; mode:tcl ***
 # ;;; End: ***
