@@ -26,6 +26,10 @@ proc registerModuleEval {context msgrecid {failedmod {}} {failedcontext {}}} {
    set evalid [topState evalid]
    set unset [expr {$failedmod eq {} ? 0 : 1}]
    set contextset 0
+   # unload dependent reload evaluations are mixed with dependent unload ones
+   if {$context eq {depre_un}} {
+      set context depun
+   }
 
    # add msgrecid to existing evaluation context list
    if {[info exists ::g_moduleEval($evalid)]} {
@@ -55,36 +59,126 @@ proc registerModuleEval {context msgrecid {failedmod {}} {failedcontext {}}} {
    # add mod to failed evaluation list
    if {$unset} {
       lappend ::g_moduleFailedEval($evalid) $failedcontext $failedmod
+      if {[depthState reportholdid]} {
+         lappend ::g_holdModuleFailedEval([currentState reportholdid])\
+            $evalid $failedcontext $failedmod
+      }
    }
 }
 
 # record that module evaluation is set hidden
 proc registerModuleEvalHidden {context msgrecid} {
    set evalid [topState evalid]
+   # unload dependent reload evaluations are mixed with dependent unload ones
+   if {$context eq {depre_un}} {
+      set context depun
+   }
    lappend ::g_moduleHiddenEval($evalid:$context) $msgrecid
 }
 
+proc changeContextOfModuleEval {mod old_context new_context} {
+   set evalid [topState evalid]
+   # find old and new context evaluations
+   for {set i 0} {$i < [llength $::g_moduleEval($evalid)]} {incr i 1} {
+      set context_eval_list [lindex $::g_moduleEval($evalid) $i]
+      if {[lindex $context_eval_list 0] eq $old_context} {
+         set old_context_idx $i
+         set old_context_eval_list $context_eval_list
+      } elseif {[lindex $context_eval_list 0] eq $new_context} {
+         set new_context_idx $i
+         set new_context_eval_list $context_eval_list
+      }
+      if {[info exists old_context_idx] && [info exists new_context_idx]} {
+         break
+      }
+   }
+
+   # module evaluation not found
+   if {![info exists old_context_idx]} {
+      return
+   }
+
+   # new context has no evaluation yet
+   if {![info exists new_context_idx]} {
+      set new_context_idx end+1
+      set new_context_eval_list [list $new_context]
+   }
+
+   # find evaluation id of module
+   foreach msgrecid [lrange $old_context_eval_list 1 end] {
+      if {$mod eq [getModuleFromEvalId $msgrecid]} {
+         set mod_evalid $msgrecid
+         break
+      }
+   }
+   if {![info exists mod_evalid]} {
+      return
+   }
+
+   # remove evaluation from old context
+   set old_context_eval_list [replaceFromList $old_context_eval_list\
+      $mod_evalid]
+   lset ::g_moduleEval($evalid) $old_context_idx $old_context_eval_list
+
+   # insert evaluation at start of new context
+   set new_context_eval_list [linsert $new_context_eval_list 1 $mod_evalid]
+   # on Tcl 8.5, lset command cannot append a list
+   if {$new_context_idx eq {end+1}} {
+      lappend ::g_moduleEval($evalid) $new_context_eval_list
+   } else {
+      lset ::g_moduleEval($evalid) $new_context_idx $new_context_eval_list
+   }
+
+   # change context of hidden evaluation
+   set old_hidden_evalid $evalid:$old_context
+   if {[info exists ::g_moduleHiddenEval($old_hidden_evalid)]} {
+      if {$mod_evalid in $::g_moduleHiddenEval($old_hidden_evalid)} {
+         set ::g_moduleHiddenEval($old_hidden_evalid) [replaceFromList\
+            $::g_moduleHiddenEval($old_hidden_evalid) $mod_evalid]
+         set new_hidden_evalid $evalid:$new_context
+         lappend ::g_moduleHiddenEval($new_hidden_evalid) $mod_evalid
+      }
+   }
+}
+
 # get context of currently evaluated module
 proc currentModuleEvalContext {} {
    return [lindex $::g_moduleEvalAttempt([currentState modulenamevr]) end]
 }
 
 # record module evaluation attempt and corresponding context
-proc registerModuleEvalAttempt {context mod} {
+proc registerModuleEvalAttempt {context mod mod_file} {
    appendNoDupToList ::g_moduleEvalAttempt($mod) $context
+   appendNoDupToList ::g_moduleFileEvalAttempt($mod) $mod_file
 }
 
-proc unregisterModuleEvalAttempt {context mod} {
+proc unregisterModuleEvalAttempt {context mod mod_file} {
    set ::g_moduleEvalAttempt($mod) [replaceFromList\
       $::g_moduleEvalAttempt($mod) $context]
+   set ::g_moduleFileEvalAttempt($mod) [replaceFromList\
+      $::g_moduleFileEvalAttempt($mod) $mod_file]
 }
 
 # is at least one module passed as argument evaluated in passed context
-proc isModuleEvaluated {context exclmod args} {
+proc isModuleEvaluated {context exclmod modulepath_list args} {
    set ret 0
    # look at all evaluated mod except excluded one (currently evaluated mod)
    foreach evalmod [lsearch -all -inline -not [array names\
       ::g_moduleEvalAttempt] $exclmod] {
+      # test evaluated module matches specified modulepaths
+      if {[llength $modulepath_list]} {
+         set eval_mod_file_match 0
+         foreach eval_mod_file $::g_moduleFileEvalAttempt($evalmod) {
+            if {[isModulefileInModulepathList $eval_mod_file\
+               $modulepath_list]} {
+               set eval_mod_file_match 1
+               break
+            }
+         }
+         if {!$eval_mod_file_match} {
+            continue
+         }
+      }
       set evalmatch 0
       # test arguments against all names of evaluated module (translate
       # eventual modspec in evalmod into module names, in case module
@@ -222,7 +316,7 @@ proc sortModulePerLoadedAndDepOrder {modlist {nporeq 0} {loading 0}} {
 
 # return list of loaded modules having an unmet requirement on passed mod
 # and their recursive dependent
-proc getUnmetDependentLoadedModuleList {modnamevr} {
+proc getUnmetDependentLoadedModuleList {modnamevr mod_file} {
    set unmetdeplist {}
    set depmodlist {}
    defineModEqProc [isIcase] [getConf extended_default] 1
@@ -230,11 +324,16 @@ proc getUnmetDependentLoadedModuleList {modnamevr} {
    set vrlist [getVariantList $modnamevr 0 0 1]
 
    # skip dependent analysis if mod has a conflict with a loaded module
-   lassign [doesModuleConflict $mod] doescon modconlist
-   if {!$doescon} {
+   set modconlist [getModuleLoadedConflict $mod]
+   if {![llength $modconlist]} {
       foreach ummod [array names ::g_unmetDepHash] {
          if {[modEq $ummod $mod eqstart 1 2 1 $vrlist]} {
-            foreach depmod $::g_unmetDepHash($ummod) {
+            foreach {depmod prereq_path_list} $::g_unmetDepHash($ummod) {
+               if {![isModulefileMatchSpecificPath $mod_file\
+                  $prereq_path_list]} {
+                  continue
+               }
+
                lappend depmodlist $depmod
                # temporarily remove prereq violation of depmod if mod
                # load solves it (no other prereq is missing)
@@ -347,11 +446,13 @@ proc getDirectDependentList {mod {strong 0} {nporeq 0} {loading 0}\
       }
       foreach loadingmod [getLoadingModuleList] {
          foreach prereq [getLoadedPrereq $loadingmod] {
+            set prereq_path_list [getLoadedPrereqPath $loadingmod $prereq]
             set lmprelist {}
             set moddep 0
             foreach modpre $prereq {
                foreach lmmod $modlist {
-                  if {[modEq $modpre $lmmod eqstart 1 2 1]} {
+                  if {[isLoadedMatchSpecificPath $lmmod $prereq_path_list]\
+                     && [modEq $modpre $lmmod eqstart 1 2 1]} {
                      lappend lmprelist $lmmod
                      if {$lmmod eq $mod} {
                         set moddep 1
@@ -403,8 +504,8 @@ proc getDependentLoadedModuleList {modlist {strong 1} {direct 1} {nporeq 0}\
       # with their own dependent modules as mod is being unloaded. Achieve so
       # by faking that conflict violation is gone
       foreach mod $modlist {
-         lassign [doesModuleConflict $mod] doescon modconlist
-         if {$doescon} {
+         set modconlist [getModuleLoadedConflict $mod]
+         if {[llength $modconlist]} {
             unsetModuleConflictViolation $mod
             set conunvioarr($mod) $modconlist
             appendNoDupToList fulllist {*}$modconlist
@@ -443,23 +544,19 @@ proc getDependentLoadedModuleList {modlist {strong 1} {direct 1} {nporeq 0}\
    return $sortlist
 }
 
-# test if passed 'mod' could be automatically unloaded or not, which means it
-# has been loaded automatically and no loaded modules require it anymore.
+# may given module be automatically unloaded
 # unmodlist: pass a list of modules that are going to be unloaded
 proc isModuleUnloadable {mod {unmodlist {}}} {
    set ret 1
-   # get currently unloading modules if no specific unmodlist set
-   if {![llength $unmodlist]} {
-      set unmodlist [getUnloadingModuleList]
-   }
 
-   # loaded tag means module was not auto-loaded. keep-loaded means module is
-   # not auto-unloadable even if auto-loaded
+   # module may be unloaded if it has been automatically loaded, it is not
+   # tagged keep-loaded and it is not sticky.
    if {[isModuleTagged $mod loaded 1] || [isModuleTagged $mod keep-loaded\
-      1]} {
+      1] || [isModuleSticky $mod]} {
       set ret 0
+   # there should no one requiring module or these dependent should be part of
+   # the unloaded/unloading list
    } else {
-      # mod is unloadable if all its dependent are unloaded or unloading
       foreach depmod [getDirectDependentList $mod] {
          if {$depmod ni $unmodlist} {
             set ret 0
@@ -471,57 +568,29 @@ proc isModuleUnloadable {mod {unmodlist {}}} {
    return $ret
 }
 
-# gets the list of all loaded modules which are required by passed modlist
-# ordered by load position.
-proc getRequiredLoadedModuleList {modlist} {
+# gets the list of all loaded modules which are required by passed mod_list
+# ordered by load position. a list of modules to exclude may be provided
+# (usually to skip requirements that will be unloaded)
+proc getRequiredLoadedModuleList {mod_list {excluded_mod_list {}}} {
    # search over all list of loaded modules, starting with passed module
    # list, then adding in turns their requirements
-   set fulllist $modlist
-   for {set i 0} {$i < [llength $fulllist]} {incr i 1} {
+   set full_list $mod_list
+   for {set i 0} {$i < [llength $full_list]} {incr i 1} {
       # gets the list of loaded modules which are required by depmod
-      appendNoDupToList fulllist {*}$::g_moduleDepend([lindex $fulllist $i])
-   }
-
-   # sort complete result list to match both loaded and dependency orders
-   set sortlist [sortModulePerLoadedAndDepOrder [lrange $fulllist [llength\
-      $modlist] end]]
-   reportDebug "got '$sortlist'"
-   return $sortlist
-}
-
-# finds required modules that can be unloaded if passed modules are unloaded:
-# they have been loaded automatically and are not depended (mandatory or
-# optionally) by other module
-proc getUnloadableLoadedModuleList {modlist} {
-   # search over all list of unloaded modules, starting with passed module
-   # list, then adding in turns unloadable requirements
-   set fulllist $modlist
-   for {set i 0} {$i < [llength $fulllist]} {incr i 1} {
-      set depmod [lindex $fulllist $i]
-      # gets the list of loaded modules which are required by depmod
-      set deplist {}
-      foreach lmmodlist $::g_moduleDepend($depmod) {
-         foreach lmmod $lmmodlist {
-            if {$lmmod ni $fulllist} {
-               lappend deplist $lmmod
+      foreach req_mod_list $::g_moduleDepend([lindex $full_list $i]) {
+         foreach req_mod $req_mod_list {
+            if {$req_mod ni $excluded_mod_list} {
+               appendNoDupToList full_list $req_mod
             }
          }
       }
-
-      # get those required module that have been automatically loaded and are
-      # only required by modules currently being unloaded
-      foreach lmmod $deplist {
-         if {[isModuleUnloadable $lmmod $fulllist]} {
-            lappend fulllist $lmmod
-         }
-      }
    }
 
    # sort complete result list to match both loaded and dependency orders
-   set sortlist [sortModulePerLoadedAndDepOrder [lrange $fulllist [llength\
-      $modlist] end]]
-   reportDebug "got '$sortlist'"
-   return $sortlist
+   set sort_list [sortModulePerLoadedAndDepOrder [lrange $full_list [llength\
+      $mod_list] end]]
+   reportDebug "got '$sort_list'"
+   return $sort_list
 }
 
 # how many settings bundle are currently saved
@@ -541,12 +610,15 @@ proc pushSettings {} {
       g_loadedModuleAliasAltname g_moduleDepend g_dependHash\
       g_moduleNPODepend g_dependNPOHash g_prereqViolation\
       g_prereqNPOViolation g_conflictViolation g_moduleUnmetDep\
-      g_unmetDepHash g_moduleEval g_moduleHiddenEval g_scanModuleVariant} {
+      g_unmetDepHash g_moduleEval g_moduleHiddenEval g_scanModuleVariant\
+      g_savedLoReqOfReloadMod g_savedLoReqOfUnloadMod\
+      g_loadedModulePrereqPath} {
       ##nagelfar ignore Suspicious variable name
       lappend ::g_SAVE_$var [array get ::$var]
    }
    # save non-array variable and indication if it was set
-   foreach var {g_changeDir g_stdoutPuts g_prestdoutPuts g_return_text} {
+   foreach var {g_changeDir g_stdoutPuts g_prestdoutPuts g_return_text\
+      g_uReqUnFromDepReList} {
       ##nagelfar ignore #4 Suspicious variable name
       if {[info exists ::$var]} {
          lappend ::g_SAVE_$var [list 1 [set ::$var]]
@@ -568,7 +640,9 @@ proc popSettings {} {
       g_loadedModuleAliasAltname g_moduleDepend g_dependHash\
       g_moduleNPODepend g_dependNPOHash g_prereqViolation\
       g_prereqNPOViolation g_conflictViolation g_moduleUnmetDep\
-      g_unmetDepHash g_moduleEval g_moduleHiddenEval g_scanModuleVariant} {
+      g_unmetDepHash g_moduleEval g_moduleHiddenEval g_scanModuleVariant\
+      g_savedLoReqOfReloadMod g_savedLoReqOfUnloadMod g_uReqUnFromDepReList\
+      g_loadedModulePrereqPath} {
       ##nagelfar ignore Suspicious variable name
       set ::g_SAVE_$var [lrange [set ::g_SAVE_$var] 0 end-1]
    }
@@ -584,7 +658,9 @@ proc restoreSettings {} {
       g_loadedModuleAliasAltname g_moduleDepend g_dependHash\
       g_moduleNPODepend g_dependNPOHash g_prereqViolation\
       g_prereqNPOViolation g_conflictViolation g_moduleUnmetDep\
-      g_unmetDepHash g_moduleEval g_moduleHiddenEval g_scanModuleVariant} {
+      g_unmetDepHash g_moduleEval g_moduleHiddenEval g_scanModuleVariant\
+      g_savedLoReqOfReloadMod g_savedLoReqOfUnloadMod\
+      g_loadedModulePrereqPath} {
       # clear current $var arrays
       ##nagelfar ignore #5 Suspicious variable name
       if {[info exists ::$var]} {
@@ -606,7 +682,8 @@ proc restoreSettings {} {
       }
    }
    # restore non-array variable if it was set
-   foreach var {g_changeDir g_stdoutPuts g_prestdoutPuts g_return_text} {
+   foreach var {g_changeDir g_stdoutPuts g_prestdoutPuts g_return_text\
+      g_uReqUnFromDepReList} {
       ##nagelfar ignore #6 Suspicious variable name
       if {[info exists ::$var]} {
          unset ::$var
@@ -635,7 +712,8 @@ proc flushEnvSettings {} {
 }
 
 # load modules passed as args designated as requirement
-proc loadRequirementModuleList {tryload optional tag_list args} {
+proc loadRequirementModuleList {tryload optional tag_list modulepath_list\
+   args} {
    set ret 0
    set prereqloaded 0
 
@@ -643,8 +721,9 @@ proc loadRequirementModuleList {tryload optional tag_list args} {
    set loadedmod_list {}
    foreach mod $args {
       # get all loaded or loading mod in args list
-      if {[set loadedmod [getLoadedMatchingName $mod returnfirst]] ne {} ||\
-         [set loadedmod [getLoadedMatchingName $mod returnfirst 1]] ne {}} {
+      if {[set loadedmod [getLoadedMatchingName $mod returnfirst 0 {}\
+         $modulepath_list]] ne {} || [set loadedmod [getLoadedMatchingName\
+         $mod returnfirst 1 {} $modulepath_list]] ne {}} {
          lappend loadedmod_list $loadedmod
       }
    }
@@ -664,7 +743,7 @@ proc loadRequirementModuleList {tryload optional tag_list args} {
          set curholdid load-$i-$arg
          lappendState reportholdid $curholdid
          if {[catch {set retlo [cmdModuleLoad reqlo 0 $tryload 0 $tag_list\
-            $arg]} errorMsg]} {
+            $modulepath_list $arg]} errorMsg]} {
             # if an error is raised, release output and rethrow the error
             # (could be raised if no modulepath defined for instance)
             lpopState reportholdid
@@ -680,7 +759,8 @@ proc loadRequirementModuleList {tryload optional tag_list args} {
          lpopState reportholdid
          lpopState reportholdrecid
 
-         if {[is-loaded $arg]} {
+         if {[string length [getLoadedMatchingName $arg returnfirst 0 {}\
+            $modulepath_list]]} {
             set prereqloaded 1
             # set previous reports to be dropped as this one succeed
             if {[info exists holdidlist]} {
@@ -705,67 +785,89 @@ proc loadRequirementModuleList {tryload optional tag_list args} {
    return [list $ret $prereqloaded]
 }
 
+# save reloading module properties before they vanish with unload phase
+proc savePropsOfReloadingModule {mod} {
+   set is_user_asked [isModuleTagged $mod loaded 1]
+   set vr_list [getVariantList $mod 1 2]
+   set tag_list [getTagList $mod]
+   set extra_tag_list [getExtraTagList $mod]
+   set conflict_list [getLoadedConflict $mod]
+   set prereq_list [getLoadedPrereq $mod]
+   set prereq_path_list [getLoadedPrereqPath $mod]
+   set ::g_savedPropsOfReloadMod($mod) [list $is_user_asked $vr_list\
+      $tag_list $extra_tag_list $conflict_list $prereq_list $prereq_path_list]
+}
+
+proc getSavedPropsOfReloadingModule {mod} {
+   return $::g_savedPropsOfReloadMod($mod)
+}
+
 # unload phase of a list of modules reload process
-proc reloadModuleListUnloadPhase {lmname {errmsgtpl {}} {context unload}} {
-   upvar $lmname lmlist
+proc reloadModuleListUnloadPhase {mod_list {err_msg_tpl {}} {context\
+   unload}} {
    # unload one by one to ensure same behavior whatever auto_handling state
-   foreach mod [lreverse $lmlist] {
-      # record hint that mod will be reloaded (useful in case mod is sticky)
-      lappendState reloading_sticky $mod
-      lappendState reloading_supersticky $mod
-      # save user asked state before it vanishes
-      set isuasked($mod) [isModuleTagged $mod loaded 1]
-      # save variants set for modules
-      set vr($mod) [getVariantList $mod 1 2]
-      # save extra tags set
-      set extratag($mod) [getExtraTagList $mod]
-      # force unload even if requiring mods are not part of the unload list
-      # (violation state) as modules are loaded again just after
-      if {[cmdModuleUnload $context match 0 1 0 0 $mod]} {
-         # avoid failing module on load phase
-         # if force state is enabled, cmdModuleUnload returns 0
-         set lmlist [replaceFromList $lmlist $mod]
-         set errMsg [string map [list _MOD_ [getModuleDesignation loaded\
-            $mod]] $errmsgtpl]
-         lpopState reloading_sticky
-         lpopState reloading_supersticky
-         # no process stop if ongoing reload command in continue behavior
-         if {![isStateEqual commandname reload] || [commandAbortOnError]} {
-            knerror $errMsg
-         }
+   foreach mod [lreverse $mod_list] {
+      if {[reloadModuleUnloadPhase $mod $err_msg_tpl $context]} {
+         set mod_list [replaceFromList $mod_list $mod]
       }
+   }
+   return $mod_list
+}
+
+proc reloadModuleUnloadPhase {mod {err_msg_tpl {}} {context unload}} {
+   # record hint that mod will be reloaded (useful in case mod is sticky)
+   lappendState reloading_sticky $mod
+   lappendState reloading_supersticky $mod
+   savePropsOfReloadingModule $mod
+   if {[set ret [cmdModuleUnload $context match 0 s 0 $mod]]} {
+      # avoid failing module on load phase
+      # if force state is enabled, cmdModuleUnload returns 0
+      set err_msg [string map [list _MOD_ [getModuleDesignation loaded $mod]]\
+         $err_msg_tpl]
       lpopState reloading_sticky
       lpopState reloading_supersticky
+      # no process stop if ongoing reload command in continue behavior
+      if {![isStateEqual commandname reload] || [commandAbortOnError]} {
+         knerror $err_msg
+      }
    }
-   return [list [array get isuasked] [array get vr] [array get extratag]]
+   lpopState reloading_sticky
+   lpopState reloading_supersticky
+   return $ret
 }
 
 # load phase of a list of modules reload process
-proc reloadModuleListLoadPhase {lmname isuaskedlist vrlist extrataglist\
-   {errmsgtpl {}} {context load}} {
-   upvar $lmname lmlist
-   array set isuasked $isuaskedlist
-   array set vr $vrlist
-   array set extratag $extrataglist
-
+proc reloadModuleListLoadPhase {mod_list {errmsgtpl {}} {context load}} {
+   if {![llength $mod_list]} {
+      return
+   }
    # loads are made with auto handling mode disabled to avoid disturbances
    # from a missing prereq automatically reloaded, so these module loads may
    # fail as prereq may not be satisfied anymore
    setConf auto_handling 0
-   foreach mod $lmlist {
+   foreach mod $mod_list {
+      lassign [getSavedPropsOfReloadingModule $mod] is_user_asked vr_list\
+         tag_list extra_tag_list conflict_list prereq_list prereq_path_list
       # if an auto set default was excluded, module spec need parsing
-      lassign [parseModuleSpecification 0 0 0 0 $mod {*}$vr($mod)] modnamevr
+      lassign [parseModuleSpecification 0 0 0 0 $mod {*}$vr_list] modnamevr
+
+      # do not try to reload DepRe module if requirements are not satisfied
+      # unless if sticky
+      if {$context eq {depre} && ![isModuleLoadable $mod $modnamevr\
+         $conflict_list $prereq_list $prereq_path_list] &&\
+         ![isModuleStickyFromTagList {*}$tag_list {*}$extra_tag_list]} {
+         continue
+      }
+
       # reload module with user asked property and extra tags preserved
-      if {[cmdModuleLoad $context $isuasked($mod) 0 0 $extratag($mod)\
+      if {[cmdModuleLoad $context $is_user_asked 0 0 $extra_tag_list {}\
          $modnamevr]} {
          set errMsg [string map [list _MOD_ [getModuleDesignation spec\
             $modnamevr]] $errmsgtpl]
          # no process stop if forced, or ongoing reload or switch cmd in
-         # continue behavior, or non-top switch cmd
+         # continue behavior
          if {[getState force] || (([isStateEqual commandname reload] ||\
-            [isStateEqual commandname switch]) && ![commandAbortOnError]) ||\
-            ([getCallingProcName] eq {cmdModuleSwitch} &&\
-            ![isTopEvaluation])} {
+            [isStateEqual commandname switch]) && ![commandAbortOnError])} {
             # no msg for reload sub-cmd which provides an empty msg template
             reportWarning $errMsg
          # stop if one load fails unless force mode enabled
@@ -777,6 +879,47 @@ proc reloadModuleListLoadPhase {lmname isuaskedlist vrlist extrataglist\
    setConf auto_handling 1
 }
 
+proc isModuleLoadable {mod mod_vr conflict_list prereq_list\
+   prereq_path_list} {
+   setLoadedConflict $mod {*}$conflict_list
+   set is_conflicting [llength [getModuleLoadedConflict $mod]]
+   unsetLoadedConflict $mod
+
+   if {$is_conflicting} {
+      return 0
+   }
+
+   array set prereq_path_arr $prereq_path_list
+   foreach prereq_arg $prereq_list {
+      if {[info exists prereq_path_arr($prereq_arg)]} {
+         set prereq_arg_path $prereq_path_arr($prereq_arg)
+      } else {
+         set prereq_arg_path {}
+      }
+      set is_requirement_loaded 0
+      foreach req_mod $prereq_arg {
+         # is requirement loaded, loading or optional
+         if {[string length [getLoadedMatchingName $req_mod returnfirst 0 {}\
+            $prereq_arg_path]] || [string length [getLoadedMatchingName\
+            $req_mod returnfirst 1 {} $prereq_arg_path]] || $req_mod eq\
+            $mod} {
+            set is_requirement_loaded 1
+            break
+         }
+      }
+      if {!$is_requirement_loaded} {
+         return 0
+      }
+   }
+
+   return 1
+}
+
+proc isModuleStickyFromTagList {args} {
+   return [expr {{super-sticky} in $args || ({sticky} in $args && ![getState\
+      force])}]
+}
+
 # test if loaded module 'mod' is sticky and if stickiness definition applies
 # to one of the reloading module
 proc isStickinessReloading {mod reloading_mod_list {tag sticky}} {
@@ -828,6 +971,224 @@ proc isStickinessReloading {mod reloading_mod_list {tag sticky}} {
    return $res
 }
 
+proc isModuleSticky {mod} {
+   return [expr {[isModuleTagged $mod super-sticky 1] || ([isModuleTagged\
+      $mod sticky 1] && ![getState force])}]
+}
+
+proc saveLoadedReqOfUnloadingModule {unload_mod} {
+   # fetch requirements of unloading module
+   set ::g_savedLoReqOfUnloadMod($unload_mod)\
+      [getRequiredLoadedModuleList [list $unload_mod]]
+}
+
+proc getLoadedReqOfUnloadingModuleList {} {
+   set unloading_req_mod_list {}
+   foreach unloading_mod [array names ::g_savedLoReqOfUnloadMod] {
+      appendNoDupToList unloading_req_mod_list\
+         {*}$::g_savedLoReqOfUnloadMod($unloading_mod)
+   }
+   return [sortModulePerLoadedAndDepOrder $unloading_req_mod_list]
+}
+
+proc clearLoadedReqOfUnloadingModuleList {} {
+   array unset ::g_savedLoReqOfUnloadMod
+}
+
+proc saveLoadedReqOfReloadingModuleList {reload_mod_list unload_mod_list} {
+   # fetch requirements of reloading modules skipping unloading ones
+   foreach reload_mod $reload_mod_list {
+      set ::g_savedLoReqOfReloadMod($reload_mod) [getRequiredLoadedModuleList\
+         [list $reload_mod] $unload_mod_list]
+   }
+}
+
+proc getLoadedReqOfReloadingModuleList {} {
+   set reloading_req_mod_list {}
+   foreach reloading_mod [array names ::g_savedLoReqOfReloadMod] {
+      appendNoDupToList reloading_req_mod_list\
+         {*}$::g_savedLoReqOfReloadMod($reloading_mod)
+   }
+   return [sortModulePerLoadedAndDepOrder $reloading_req_mod_list]
+}
+
+proc unsetLoadedReqOfReloadingModule {mod} {
+   unset -nocomplain ::g_savedLoReqOfReloadMod($mod)
+}
+
+proc clearLoadedReqOfReloadingModuleList {} {
+   array unset ::g_savedLoReqOfReloadMod
+}
+
+proc identityUReqUnFromDepRe {depre_list unload_mod_list} {
+   foreach depre [lreverse $depre_list] {
+      if {[isModuleUnloadable $depre $unload_mod_list]} {
+         lappend unload_mod_list $depre
+         lappend ::g_uReqUnFromDepReList $depre
+      }
+   }
+}
+
+proc clearUReqUnFromDepReList {} {
+   unset -nocomplain ::g_uReqUnFromDepReList
+}
+
+proc getIdentifiedUReqUnFromDepRe {} {
+   if {[info exists ::g_uReqUnFromDepReList]} {
+      return $::g_uReqUnFromDepReList
+   }
+}
+
+proc removeUReqUnFromDepReAndConvertEval {} {
+   set depre_list [getDepReList]
+   set urequn_from_depre_list [getIdentifiedUReqUnFromDepRe]
+   lassign [getDiffBetweenList $depre_list $urequn_from_depre_list] depre_list
+   setDepReList $depre_list
+   foreach urequn_from_depre [lreverse $urequn_from_depre_list] {
+      changeContextOfModuleEval $urequn_from_depre depun urequn
+   }
+}
+
+proc getUReqUnModuleList {} {
+   set unloadable_mod_list {}
+
+   # add DepRe modules that are UReqUn modules and unset their requirements
+   # from reloading list (to check them for potential UReqUn modules)
+   foreach urequn_from_depre [getIdentifiedUReqUnFromDepRe] {
+      lappend unloadable_mod_list $urequn_from_depre
+      unsetLoadedReqOfReloadingModule $urequn_from_depre
+   }
+
+   set reloading_req_mod_list [getLoadedReqOfReloadingModuleList]
+   # useless requirement unload modules are unloadable req of unloaded mods
+   # treat lastly loaded module first to build unloadable module list
+   foreach unloading_req_mod [lreverse [getLoadedReqOfUnloadingModuleList]] {
+      if {$unloading_req_mod ni $reloading_req_mod_list &&\
+         [isModuleUnloadable $unloading_req_mod $unloadable_mod_list]} {
+         lappend unloadable_mod_list $unloading_req_mod
+      }
+   }
+   # return result in loaded order
+   return [lreverse $unloadable_mod_list]
+}
+
+proc unloadUReqUnModules {} {
+   set urequn_list [getUReqUnModuleList]
+   reportDebug "urequn mod list is '$urequn_list'"
+
+   if {[llength $urequn_list]} {
+      # DepRe: Dependent to Reload (modules optionally dependent or in
+      # conflict with modname, DepUn or UReqUn modules + modules dependent of
+      # a module part of this DepRe batch)
+      set urequn_depre_list [getDependentLoadedModuleList $urequn_list 0 0 1\
+         0 1 1]
+      set urequn_depre_list [reloadModuleListUnloadPhase $urequn_depre_list\
+         {Unload of dependent _MOD_ failed} depre_un]
+      lprependDepReList $urequn_depre_list
+
+      set urequn_list [lreverse $urequn_list]
+      for {set i 0} {$i < [llength $urequn_list]} {incr i 1} {
+         set unmod [lindex $urequn_list $i]
+         if {[cmdModuleUnload urequn match 0 s 0 $unmod]} {
+            # main unload process continues, but the UReqUn modules that are
+            # required by unmod (whose unload failed) are withdrawn from
+            # UReqUn module list
+            lassign [getDiffBetweenList $urequn_list\
+               [getRequiredLoadedModuleList [list $unmod]]] urequn_list
+         }
+      }
+   }
+}
+
+proc getDepUnModuleList {mod} {
+   set depun_npo_list [getDependentLoadedModuleList [list $mod] 1 0 1 0]
+   set depun_list [getDependentLoadedModuleList [list $mod] 1 0 0 0]
+   # look at both regular dependencies or No Particular Order dependencies:
+   # use NPO result if situation can be healed with NPO dependencies, which
+   # will be part of DepRe list to restore the correct loading order for them
+   if {[llength $depun_npo_list] <= [llength $depun_list]} {
+      set depun_list $depun_npo_list
+   }
+   return $depun_list
+}
+
+proc clearDepReList {} {
+   unset -nocomplain ::g_depReList
+}
+
+proc getDepReList {} {
+   if {[info exists ::g_depReList]} {
+      return $::g_depReList
+   }
+}
+
+proc setDepReList {mod_list} {
+   set ::g_depReList $mod_list
+}
+
+proc lprependDepReList {mod_list} {
+   lprepend ::g_depReList {*}$mod_list
+}
+
+proc unloadDepUnDepReModules {unload_mod_list reload_mod_list} {
+   set err_msg_tpl {Unload of dependent _MOD_ failed}
+   set unload_mod_list [sortModulePerLoadedAndDepOrder [list\
+      {*}$unload_mod_list {*}$reload_mod_list]]
+   lprependDepReList $reload_mod_list
+
+   foreach unload_mod [lreverse $unload_mod_list] {
+      if {$unload_mod in $reload_mod_list} {
+         # an error is raised if an unload eval fails
+         reloadModuleUnloadPhase $unload_mod $err_msg_tpl depre_un
+      } else {
+         if {[cmdModuleUnload depun match 0 s 0 $unload_mod]} {
+            # stop if one unload fails unless force mode enabled
+            set err_msg [string map [list _MOD_ [getModuleDesignation loaded\
+               $unload_mod]] $err_msg_tpl]
+            knerrorOrWarningIfForced $err_msg
+         }
+      }
+   }
+}
+
+proc reloadDepReModules {} {
+   set err_msg_tpl {Reload of dependent _MOD_ failed}
+   set depre_list [getDepReList]
+   reloadModuleListLoadPhase $depre_list $err_msg_tpl depre
+}
+
+# Fail unload attempt if module is sticky, unless if forced or reloading
+# Also fail unload if mod is super-sticky even if forced, unless reloading
+proc failOrSkipUnloadIfSticky {modname modfile} {
+   # when loaded, tags applies to mod name and version (not with variant)
+   set is_supersticky_not_reloading [expr {[isModuleTagged $modname\
+      super-sticky 1 $modfile] && [currentState reloading_supersticky] ne\
+      $modname}]
+   set is_sticky_not_reloading [expr {[isModuleTagged $modname sticky 1\
+      $modfile] && [currentState reloading_sticky] ne $modname &&\
+      [currentState unloading_sticky] ne $modname}]
+   set sticky_purge [expr {[getState commandname] eq {purge} ? [getConf\
+      sticky_purge] : {}}]
+
+   if {!$is_supersticky_not_reloading && $is_sticky_not_reloading &&\
+      [getState force]} {
+      reportWarning [getStickyForcedUnloadMsg]
+   } elseif {$is_supersticky_not_reloading || $is_sticky_not_reloading} {
+      set msg [getStickyUnloadMsg [expr {$is_supersticky_not_reloading ?\
+         {super-sticky} : {sticky}}]]
+      # no message if sticky_purge is set to silent
+      switch -- $sticky_purge {
+         error - {} {knerror $msg}
+         warning {reportWarning $msg}
+      }
+
+      # skip unload without raising error
+      return 1
+   }
+
+   return 0
+}
+
 # ;;; Local Variables: ***
 # ;;; mode:tcl ***
 # ;;; End: ***
