@@ -455,7 +455,7 @@ proc modVariantCmp {pvrlist modvrlist {missmean 0}} {
       # modulefile and mod is extra specifier defined on command line
    }
    foreach pvr $pvrlist {
-      set pvrarr([lindex $pvr 0]) [lindex $pvr 2]
+      set pvrarr([lindex $pvr 0]) [lindex $pvr 3]
    }
 
    # no match if a specified variant is not found among module variants (and
@@ -1324,46 +1324,53 @@ proc parseModuleSpecificationProcAdvVersSpec {mlspec nonamespec xtspec\
       if {[regexp {^[a-z-]+:} $arg]} {
          lappend curarglist $arg
       } else {
+         set split_chars {@ ~ +}
+         lappend split_chars {*}[array names ::g_shortcutVariant]
+         set split_arg_list {}
          set previ 0
          for {set i 1} {$i < [string length $arg]} {incr i} {
-            set c [string index $arg $i]
-            switch -- $c {
-               @ - ~ {
-                  lappend curarglist [string range $arg $previ $i-1]
-                  set previ $i
-               }
-               + {
-                  # allow one or more '+' char at end of module name if not
-                  # followed by non-special character (@, ~ or /)
-                  set nexti [expr {$i + 1}]
-                  if {$nexti < [string length $arg]} {
-                     switch -- [string index $arg $nexti] {
-                        @ - + - ~ - / {}
-                        default {
-                           lappend curarglist [string range $arg $previ $i-1]
-                           set previ $i
-                        }
-                     }
-                  }
-               }
-               default {
-                  # check if a variant shortcut matches
-                  if {[info exists ::g_shortcutVariant($c)]} {
-                     lappend curarglist [string range $arg $previ $i-1]
-                     set previ $i
-                  }
-               }
+            if {[string index $arg $i] in $split_chars} {
+               lappend split_arg_list [string range $arg $previ $i-1]
+               set previ $i
             }
          }
-         lappend curarglist [string range $arg $previ $i-1]
+         lappend split_arg_list [string range $arg $previ $i-1]
+
+         # unsplit some arg parts: those starting with boolean variant prefix
+         # (+ or ~) but not followed by a valid variant name
+         set prev_arg [lindex $split_arg_list 0]
+         for {set i 1} {$i < [llength $split_arg_list]} {incr i} {
+            set split_arg [lindex $split_arg_list $i]
+            if {[string index $split_arg 0] ni {+ ~} || [isVariantNameValid\
+               [string range $split_arg 1 end]]} {
+               lappend curarglist $prev_arg
+               set prev_arg $split_arg
+            } else {
+               append prev_arg $split_arg
+            }
+         }
+         lappend curarglist $prev_arg
       }
 
       # parse each specification element
       foreach curarg $curarglist {
          set vrisbool 0
+
+         set xtnot 0
+         if {[string equal -length 4 $curarg not:]} {
+            if {!$xtspec} {
+               knerror "No extra specification allowed on this command"
+            }
+            set xtnot 1
+            set curarg [string range $curarg 4 end]
+         }
+
          set c [string index $curarg 0]
          switch -- $c {
             @ {
+               if {$xtnot} {
+                  knerror "Invalid extra specification '$arg'"
+               }
                set modspec [string range $curarg 1 end]
                lassign [parseModuleVersionSpecifier $modspec] cmpspec versspec
                continue
@@ -1418,7 +1425,7 @@ proc parseModuleSpecificationProcAdvVersSpec {mlspec nonamespec xtspec\
                   knerror "Invalid extra specifier '$xtelt'\nValid extra\
                      specifiers are: $xtelt_valid_list"
                }
-               set spec_xt [list $xtelt]
+               set spec_xt [list $xtelt $xtnot]
                # parse and resolve module spec set as extra specifier value
                if {$xtelt in $xtelt_modspec_list} {
                   foreach xtname $xtnamelist {
@@ -1490,9 +1497,12 @@ proc parseModuleSpecificationProcAdvVersSpec {mlspec nonamespec xtspec\
                }
                # save variant name and value
                set vrnamearr($vrname) $vridx
-               lappend vrlist [list $vrname $vrisbool {*}$vrvaluelist]
+               lappend vrlist [list $vrname $xtnot $vrisbool {*}$vrvaluelist]
             }
             default {
+               if {$xtnot} {
+                  knerror "Invalid extra specification '$arg'"
+               }
                # save previous mod version spec and transformed arg if any
                if {[info exists modarglist]} {
                   set modarg [join $modarglist]
@@ -1662,21 +1672,19 @@ proc unsetModuleVersSpec {modarg} {
 # get module name from module name and version spec if parsed
 proc getModuleNameFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
-      lassign $::g_moduleVersSpec($modarg) mod modname
+      return [lindex $::g_moduleVersSpec($modarg) 1]
    } else {
-      set modname [file dirname $modarg]
+      return [file dirname $modarg]
    }
-   return $modname
 }
 
 # get module root name from module name and version spec if parsed
 proc getModuleRootFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
-      set modroot [lindex $::g_moduleVersSpec($modarg) 6]
+      return [lindex $::g_moduleVersSpec($modarg) 6]
    } else {
-      set modroot [lindex [file split $modarg] 0]
+      return [lindex [file split $modarg] 0]
    }
-   return $modroot
 }
 
 # translate module name version spec to return all modules mentioned
@@ -1727,42 +1735,34 @@ proc getOneModuleFromVersSpec {modarg} {
 # translate module name version spec to return the list of variant mentioned
 proc getVariantListFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
-      set variantlist [lindex $::g_moduleVersSpec($modarg) 7]
-   } else {
-      set variantlist {}
+      return [lindex $::g_moduleVersSpec($modarg) 7]
    }
-   return $variantlist
 }
 
 # get module name and version from version spec if parsed
 proc getModuleNameAndVersFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
-      set modnvspec [lindex $::g_moduleVersSpec($modarg) 8]
+      return [lindex $::g_moduleVersSpec($modarg) 8]
    } else {
-      set modnvspec $modarg
+      return $modarg
    }
-   return $modnvspec
 }
 
 # get raw argument specified from parsed version spec
 proc getRawArgumentFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
-      set rawarg [lindex $::g_moduleVersSpec($modarg) 9]
+      return [lindex $::g_moduleVersSpec($modarg) 9]
    } else {
-      set rawarg $modarg
+      return $modarg
    }
-   return $rawarg
 }
 
 # translate module name version spec to return the list of extra specifier
 # mentioned
 proc getExtraListFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
-      set extralist [lindex $::g_moduleVersSpec($modarg) 10]
-   } else {
-      set extralist {}
+      return [lindex $::g_moduleVersSpec($modarg) 10]
    }
-   return $extralist
 }
 
 # translate module name version spec to return list of tag specifier and list
@@ -1785,11 +1785,8 @@ proc getSplitExtraListFromVersSpec {modarg} {
 # get available modules matching module specification
 proc getAvailListFromVersSpec {modarg} {
    if {[info exists ::g_moduleVersSpec($modarg)]} {
-      set availlist [lindex $::g_moduleVersSpec($modarg) 11]
-   } else {
-      set availlist {}
+      return [lindex $::g_moduleVersSpec($modarg) 11]
    }
-   return $availlist
 }
 
 # is mod spec containing variant but no module name and version
